@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from pprint import pprint
 import os.path
 import sys
 import copy
@@ -80,13 +79,32 @@ class CLGError(Exception):
     pass
 
 
+class Namespace(argparse.Namespace):
+    def __init__(self, namespace):
+        argparse.Namespace.__init__(self)
+        self.__dict__.update(namespace.__dict__)
+
+
+    def __getitem__(self, key):
+        return self.__dict__[key]
+
+
+    def __setitem__(self, key, value):
+        if key not in self.__dict__:
+            raise KeyError(key)
+        self.__dict__[key] = value
+
+
+    def __iter__(self):
+        return self.__dict__.iteritems()
+
+
 class CommandLine(object):
     """Command line"""
     def __init__(self, config, keyword='command'):
         """Initialize the command from a YAML or a JSON file."""
         self.config = config
         self.keyword = keyword
-        self.args = None
         self.parser = None
         self.__parsers = {}
 
@@ -167,11 +185,15 @@ class CommandLine(object):
                 continue
 
             value = {
-                'type': lambda: eval(value),
                 'help': lambda: (
                     value.replace('$DEFAULT', str(config['default']))
                         if "$DEFAULT" in str(value)
                         else value
+                ),
+                'default': lambda: (
+                    value.replace('__FILE__', sys.path[0])
+                        if type(value) in (str, unicode)
+                        else value
                 )
             }.get(param, lambda: value)()
             option_kwargs.setdefault(param, value)
@@ -221,11 +243,8 @@ class CommandLine(object):
 
         # Check 'subparsers' section is alone.
         if 'subparsers' in config and len(config) > 1:
-            raise CLGError(
-                "/%s: 'subparsers' section is in conflict with '%s'" % (
-                    '/'.join(config_path), keyword
-                )
-            )
+            raise CLGError("/%s: 'subparsers' section must be alone" % (
+                '/'.join(config_path)))
 
 
     def __add_parser(self, config_path, parent=None):
@@ -247,7 +266,7 @@ class CommandLine(object):
             subparsers_config = config.get('subparsers', {})
             subparser_dest = '%s%d' % (self.keyword, len(config_path) / 2)
             subparsers = parent.add_subparsers(dest=subparser_dest)
-            for subparser_name, subparser_conf in subparsers_config.iteritems():
+            for subparser_name in subparsers_config:
                 subparser_path = list(config_path)
                 subparser_path.extend(['subparsers', subparser_name])
                 subparser = subparsers.add_parser(subparser_name, add_help=True)
@@ -282,7 +301,7 @@ class CommandLine(object):
                 raise CLGError('/%s: %s' % ('/'.join(config_path), err))
 
 
-    def __check_dependency(self, config, option, parser, dependency=False):
+    def __check_dependency(self, args, config, option, parser, dependency=False):
         """
         dependency is True = check for needed options
         dependency is False = check for confict options
@@ -292,8 +311,8 @@ class CommandLine(object):
 
         for opt in options[option][keyword]:
             if (
-              (dependency and not self.args[opt])
-              or (not dependency and self.args[opt])
+              (dependency and not args[opt])
+              or (not dependency and args[opt])
             ):
                 parser.print_usage()
                 kwargs = {
@@ -308,20 +327,31 @@ class CommandLine(object):
                 sys.exit(1)
 
 
-    def __execute(self, exec_config):
+    def __execute(self, exec_config, args):
         if 'module' in exec_config:
-            module_config = exec_config['module']
-            exec('import %s as module' % module_config['lib'])
-            exec('module.%s(self.args)' % module_config['function'])
+            import imp
+            module_path = exec_config['module']['path']
+            search_params = ([os.path.splitext(module_path)[0]]
+                if not os.path.isabs(module_path)
+                else [
+                    os.path.splitext(os.path.basename(module_path))[0],
+                    [os.path.dirname(module_path)]
+                ]
+            )
+            module_params = imp.find_module(*search_params)
+            module = imp.load_module(
+                os.path.splitext(module_path)[0], *module_params
+            )
+            getattr(module, exec_config['module'].get('function', 'main'))(args)
 
 
-    def parse(self):
+    def parse(self, args=None):
         # Parse arguments.
-        self.args = self.parser.parse_args().__dict__
+        args = Namespace(self.parser.parse_args(args))
 
         # Get subparser configuration.
         path = []
-        for arg, value in sorted(self.args.iteritems()):
+        for (arg, value) in sorted(args):
             if arg.startswith(self.keyword):
                 path.extend([value, 'subparsers'])
         if path:
@@ -331,26 +361,25 @@ class CommandLine(object):
 
         # Post checks.
         for option, option_config in config['options'].iteritems():
-            if self.args[option] is None:
+            if args[option] is None:
+                if 'type' in option_config and option_config['type'] == 'list':
+                    args[option] = []
                 continue
 
             for keyword in POST_KEYWORDS:
                 if keyword in option_config:
                     {
                         'need': lambda:
-                            self.__check_dependency(config, option, parser, True),
+                            self.__check_dependency(
+                                args, config, option, parser, True),
                         'conflict': lambda:
-                            self.__check_dependency(config, option, parser, False)
+                            self.__check_dependency(
+                                args, config, option, parser, False)
                     }.get(keyword)()
 
-            if 'type' in option_config and option_config['type'] == 'path':
-                self.args[option] = os.path.abspath(
-                    self.args[option].replace(
-                        '__FILE__', os.path.dirname(self.conf_file)
-                    )
-                )
-
         # Execute.
         if 'execute' in config:
-            self.__execute(config['execute'])
+            self.__execute(config['execute'], args)
+
+        return args
 
